/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "nfaclosure.h"

NFAClosure::NFAClosure(const NFA& object)
:	m_nstates(0)
{
	m_moves.push_back(list<NFAMove>());
	m_moves.push_back(list<NFAMove>());
	m_nstates = 2;
	list<NFAMove> start_moves = object.getMovesFrom(START_STATE);
	list<NFAMove>::iterator iter = start_moves.begin();
	for (; iter != start_moves.end(); iter++) {
		if (iter->m_move_to == FINISH_STATE) {
			iter->m_move_to = 1;
		} else {
			iter->m_move_to += m_nstates;
		}
	}
	m_moves[0] = start_moves;
	int nstates = object.nStates();
	for (int i = 0; i < nstates; i++) {
		list<NFAMove> moves = object.getMovesFrom(i);
		iter = moves.begin();
		for (; iter != moves.end(); iter++) {
			if (iter->m_move_to == FINISH_STATE) {
				iter->m_move_to = 1;
			} else {
				iter->m_move_to += m_nstates;
			}
		}
		m_moves.push_back(moves);
	}
	m_nstates += nstates;
	m_start_moves.push_back(NFAMove(0));
	m_start_moves.push_back(NFAMove(FINISH_STATE));
	m_moves[1].push_back(NFAMove(0));
	m_moves[1].push_back(NFAMove(FINISH_STATE));
	
}

list<NFAMove>
NFAClosure::getMovesFrom(int state) const
{
	if (state == FINISH_STATE) {
		return list<NFAMove>();
	} else if (state == START_STATE) {
		return m_start_moves;
	} else {
		return m_moves[state];
	}
}

