/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "nfaconcatenation.h"

NFAConcatenation::NFAConcatenation(const NFA &first)
:	m_nstates(0), m_length(-1)
{
	addFirstComponent(first);
}

NFAConcatenation::NFAConcatenation(const NFA &first, const NFA &second)
:	m_nstates(0), m_length(-1)
{
	addFirstComponent(first);
	addComponent(second);
}

void
NFAConcatenation::addFirstComponent(const NFA& component)
{
	int nstates = component.nStates();
	for (int i = 0; i < nstates; i++) {
		m_moves.push_back(component.getMovesFrom(i));
	}
	m_start_moves = component.getMovesFrom(START_STATE);
	m_nstates = nstates;
	m_length = component.length();
}

void
NFAConcatenation::addComponent(const NFA& component)
{
	int intermediate_state = m_nstates++;
	m_moves.push_back(list<NFAMove>());
	list<NFAMove>::iterator iter = m_start_moves.begin();
	for (; iter != m_start_moves.end(); iter++) {
		if (iter->m_move_to == FINISH_STATE) {
			iter->m_move_to = intermediate_state;
		}
	}
	for (int i = 0; i < intermediate_state; i++) {
		list<NFAMove>& moves = m_moves[i];
		iter = moves.begin();
		for (; iter != moves.end(); iter++) {
			if (iter->m_move_to == FINISH_STATE) {
				iter->m_move_to = intermediate_state;
			}
		}
	}
	
	int nstates = component.nStates();
	for (int i = 0; i < nstates; i++) {
		list<NFAMove> moves = component.getMovesFrom(i);
		iter = moves.begin();
		for (; iter != moves.end(); iter++) {
			if (iter->m_move_to != FINISH_STATE)
				iter->m_move_to += m_nstates;
		}
		m_moves.push_back(moves);
	}
	list<NFAMove> moves = component.getMovesFrom(START_STATE);
	iter = moves.begin();
	for (; iter != moves.end(); iter++) {
		if (iter->m_move_to != FINISH_STATE)
			iter->m_move_to += m_nstates;
	}
	m_moves[intermediate_state] = moves;
	m_nstates += nstates;
	if (m_length == -1 || component.length() == -1) {
		m_length = -1;
	} else {
		m_length += component.length();
	}
}

list<NFAMove>
NFAConcatenation::getMovesFrom(int state) const
{
	if (state == FINISH_STATE) {
		return list<NFAMove>();
	} else if (state == START_STATE) {
		return m_start_moves;
	} else {
		return m_moves[state];
	}
}

