/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef CLIENTREQUESTSTREAM_H_
#define CLIENTREQUESTSTREAM_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include "httpstream.h"
#include "httpstreamwriter.h"
#include "httpclientrequest.h"
#include "uri.h"

using namespace std;

class BFilter;

class ClientRequestStream : public HttpStream, protected HttpStreamWriter::Receiver
{
public:
	ClientRequestStream(BFilter& owner) : m_owner(owner), m_stream_writer(0) {}
	virtual ~ClientRequestStream();
	void dispatchMoreRequests(); // to be called from BFilter, ServerResponseStream and locally
protected:
	enum ErrorCode2 {
		ERR2_UNSUPPORTED_METHOD = HttpStream::NUMBER_OF_ERRORS,
		ERR2_BAD_REQUEST_URI,
		ERR2_UNSUPPORTED_EXPECTATION
	};
	/* Inherited from HttpStream */
	virtual bool isRequestStream() const { return true; }
	virtual bool isResponseOnHeadRequest() const { return false; }
	virtual void eventError(int ecode, const char *info=0);
	virtual void eventGotMetadata(const HttpRequestMetadata& metadata);
	virtual void eventGotContentPart(const string& data);
	virtual void eventGotFooters(const HttpHeadersCollection& footers) {}
	virtual void eventMessageEnds(bool connection_needs_closing);
	
	/* Inherited from HttpStreamWriter::Receiver */
	virtual void moreData(const string& data);
	virtual void done(bool conn_close);
private:
	ClientRequestStream(const ClientRequestStream& stream);            //
	ClientRequestStream& operator=(const ClientRequestStream& stream); // not implemented
	virtual void eventGotMetadata(const HttpResponseMetadata& metadata) {}
	static void prepareRequestForForwarding(HttpRequestMetadata& metadata);
	static bool isSameServer(const URI& one, const URI& other);
	BFilter& m_owner;
	HttpStreamWriter *m_stream_writer;
};

#endif

