/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "compressorresponsefilter.h"
#include "deflatecompressor.h"
#include "gzipcompressor.h"

CompressorResponseFilter::CompressorResponseFilter(compr_type type)
:	m_type(type), m_metadata(0), m_eof(false)
{
	if (type == C_DEFLATE) {
		m_delegate = new DeflateCompressor();
	} else {
		m_delegate = new GzipCompressor();
	}
}

CompressorResponseFilter::~CompressorResponseFilter()
{
	delete m_delegate;
}

void
CompressorResponseFilter::consumeMetadata(HttpResponseMetadata& metadata)
{
	m_metadata = &metadata;
	string enctype = (m_type == C_DEFLATE ? "deflate" : "gzip");
	if (m_metadata->headers().hasHeader("Transfer-Encoding")) {
		m_metadata->headers().getHeaderPtr("Transfer-Encoding")->elements().appendElement(HttpHeaderElement(enctype));
	} else {
		if (!m_metadata->headers().hasHeader("Content-Encoding")) {
			m_metadata->headers().setHeader(HttpHeader("Content-Encoding", enctype));
		} else {
			m_metadata->headers().getHeaderPtr("Content-Encoding")->elements().appendElement(HttpHeaderElement(enctype));
		}
	}
	m_metadata->setBodyStatus(HttpMessageMetadata::BODY_SIZE_UNKNOWN);
}

void
CompressorResponseFilter::consumeDataChunk(const string& data)
{
	m_delegate->consume(data);
	string out;
	m_delegate->readOutput(out);
	m_processed_data += out;
}

void
CompressorResponseFilter::consumeEOF()
{
	m_delegate->consumeEOF();
	string out;
	m_delegate->readOutput(out);
	m_processed_data += out;
	m_eof = true;
}

bool
CompressorResponseFilter::hasDataAvailable() const
{
	return !m_processed_data.empty();
}

void
CompressorResponseFilter::readDataChunk(string& collector)
{
	collector.assign(m_processed_data);
	m_processed_data.erase();
}

