/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <cstdio>
#include <cstdlib>
#include <sstream>
#include <map>
#include <string>
#include <conflexer.h>
#include <confparser.h>
#include <conflistener.h>
#include "errorreporter.h"
#include "utils.h"
#include "conf.h"

using namespace std;

#define DEFAULT_LISTEN_ADDR	""
#define DEFAULT_LISTEN_PORT	8080

Conf Conf::m_instance;
string Conf::m_conf_dir;

Conf::Conf()
:	m_listen_addr(DEFAULT_LISTEN_ADDR), m_listen_port(DEFAULT_LISTEN_PORT),
	m_use_proxy(false), m_proxy_host(""), m_proxy_port(-1), m_client_compression(false),
	m_draw_borders(true)
{
	m_border_color[0] = 0x67;
	m_border_color[1] = 0x67;
	m_border_color[2] = 0x67;
}

bool
Conf::readConfig()
{
	const string CONFIG_FILE = m_conf_dir+"/config";
	FILE *fp = fopen(CONFIG_FILE.c_str(), "r");
	if (!fp) {
		ErrorReporter::fatalError(string("Could not open ")+CONFIG_FILE);
	}
	ConfLexer::reset();
	ConfParser::reset();
	ConfParser::setFileName(CONFIG_FILE);
	ConfParser::attachListener(this);
	bool res = ConfParser::parse(fp);
	ConfParser::detachListener(this);
	fclose(fp);
	if (m_use_proxy && m_proxy_port == -1) {
		ErrorReporter::fatalError("Proxy port not given in config");
	}
	return res;
}

void
Conf::processSection(const string& section, int line)
{
	if (section != "forwarding" && section != "global") {
		warning(string("Unknown section: ")+section, line);
	}
}

void
Conf::processParameter(const string& section, const string& key, const string& value, int line)
{
	if (section.empty() || section == "global") {
		if (key == "listen_address") {
			string::size_type pos = value.find(":");
			m_listen_addr = value.substr(0, pos);
			if (pos != string::npos) {
				istringstream strm(value.substr(pos+1));
				strm >> m_listen_port;
			}
		} else if (key == "client_compression") {
			m_client_compression = (value == "yes");
		} else if (key == "ad_border") {
			if (value == "none") {
				m_draw_borders = false;
			} else {
				m_draw_borders = true;
				if (value.length() >= 6) {
					istringstream strm1(value.substr(0, 2));
					istringstream strm2(value.substr(2, 2));
					istringstream strm3(value.substr(4, 2));
					unsigned int r = m_border_color[0], g = m_border_color[1], b = m_border_color[2];
					// just doing strm1 >> hex >> m_border_color[0] doesn't work for some reason
					strm1 >> hex >> r;
					strm2 >> hex >> g;
					strm3 >> hex >> b;
					m_border_color[0] = r;
					m_border_color[1] = g;
					m_border_color[2] = b;
				}
			}
		} else {
			warning(string("Unknown paramenter: ")+key, line);
		}
	} else {
		if (section != "forwarding") {
			return;
		}
		if (key == "use_proxy") {
			if (value == "yes")
				m_use_proxy = true;
		} else if (key == "proxy_host") {
			m_proxy_host = value;
		} else if (key == "proxy_port") {
			istringstream strm(value);
			strm >> m_proxy_port;
		} else {
			warning(string("Unknown parameter: ")+key, line);
		}
	}
}

void
Conf::processError(const string& errmsg, int line)
{
	ErrorReporter::fatalError(string("File ")+ConfParser::getFileName()+"[line "+num2str(line)+"]: "+errmsg);
}

void
Conf::warning(const string& msg, int line)
{
	ErrorReporter::warning(string("File ")+ConfParser::getFileName()+" [line "+num2str(line)+"]: "+msg);
}

