/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef CONF_H_
#define CONF_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <conflistener.h>

using namespace std;

class Conf : protected ConfListener
{
public:
	bool readConfig();
	static void setConfDir(const string& conf_dir) { m_conf_dir = conf_dir; }
	static Conf& getInstance() { return m_instance; }
	string getListenAddress() const { return m_listen_addr; }
	int getListenPort() const { return m_listen_port; }
	bool usingProxy() const { return m_use_proxy; }
	const string& proxyHost() const { return m_proxy_host; }
	int proxyPort() const { return m_proxy_port; }
	bool clientCompression() const { return m_client_compression; }
	bool drawBorders() const { return m_draw_borders; }
	const unsigned char* getBorderColor() { return m_border_color; }
protected:
	virtual void processSection(const string& section, int line);
	virtual void processParameter(const string& section, const string& key, const string& value, int line);
	virtual void processError(const string& errmsg, int line);
private:
	void warning(const string& msg, int line);
	Conf();
	static string m_conf_dir;
	static Conf m_instance;
	string m_listen_addr;
	int m_listen_port;
	bool m_use_proxy;
	string m_proxy_host;
	int m_proxy_port;
	bool m_client_compression;
	bool m_draw_borders;
	unsigned char m_border_color[3];
};

#endif
