%option noyywrap

%{
#include <string>
#include "conflexer.h"
#include "confparser.h"
#include "conftokens.h"

using namespace std;

static int extra_newline = 0;
static string value;

int mylineno = 1;

%}

%x KEY
%x VALUE
%x VALUE_Q_NE
%x VALUE_Q_E
%x VALUE_NQ

SECTION ^[\ \t]*\[.*\][\ \t]*/[#;]|\r?\n
KEY     [[:alpha:]_][[:alnum:]_]*

%%

<INITIAL,VALUE,VALUE_NQ>[#;].*/\r?\n /* skip comments */
<INITIAL>{SECTION}		{
					string section(yytext);
					string::size_type fpos1 = section.find('[');
					string::size_type fpos2 = section.rfind(']');
					section = section.substr(fpos1+1, fpos2-fpos1-1);
					yylval = ConfLexer::storeSymbol(section);
					return TOK_SECTION;
				}
<INITIAL>=			BEGIN(VALUE); value = ""; return TOK_EQUAL;
<INITIAL>\r?\n			ConfLexer::setCurLine(ConfLexer::getCurLine()+1); return TOK_NEWLINE;
<INITIAL,VALUE>[\ \t]+		/* skip spaces */
<INITIAL>^[\ \t]*/{KEY}[\ \t]*=	BEGIN(KEY);
<KEY>{KEY}			yylval = ConfLexer::storeSymbol(yytext); BEGIN(INITIAL); return TOK_KEY;
<VALUE>\"			BEGIN(VALUE_Q_NE);
<VALUE>.|\r?\n			BEGIN(VALUE_NQ); yyless(0);
<VALUE_NQ>[\ \t]+/\r?\n		BEGIN(INITIAL); yylval = ConfLexer::storeSymbol(value); return TOK_VALUE;
<VALUE_NQ>.			value += yytext[0];
<VALUE_NQ>\r?\n			BEGIN(INITIAL); yyless(0); yylval = ConfLexer::storeSymbol(value); return TOK_VALUE;
<VALUE_Q_NE>\\			BEGIN(VALUE_Q_E);
<VALUE_Q_NE>\"			BEGIN(INITIAL); yylval = ConfLexer::storeSymbol(value); return TOK_VALUE;
<VALUE_Q_NE>\r?\n		yyerror("missing closing quote"); return 0;
<VALUE_Q_NE>.			value += yytext[0];
<VALUE_Q_E>.|\n			{
					switch (yytext[0]) {
					case 'a':
						value += '\a'; break;
					case 'b':
						value += '\b'; break;
					case 'f':
						value += '\f'; break;
					case 'n':
						value += '\n'; break;
					case 'r':
						value += '\r'; break;
					case 't':
						value += '\t'; break;
					case 'v':
						value += '\v'; break;
					default:
						value += yytext[0];
						break;
					}
				}
<*>.|\n				return TOK_OTHER;
<<EOF>>				if ((extra_newline=~extra_newline) & 1) unput('\n'); else return 0;

%%

ConfLexer::symtab_type ConfLexer::m_symtab;
ConfLexer::reverse_symtab_type ConfLexer::m_reverse_symtab;
int ConfLexer::m_lineno = 1;

void
ConfLexer::setInput(FILE *input)
{
	yyin = input;
}

int
ConfLexer::storeSymbol(const string& symbol)
{
	reverse_symtab_type::const_iterator fit = m_reverse_symtab.find(symbol);
	if (fit != m_reverse_symtab.end()) {
		return fit->second;
	}
	int new_id = 0;
	if (!m_symtab.empty())
		new_id = m_symtab.rbegin()->first + 1;
	m_symtab.insert(symtab_type::value_type(new_id, symbol));
	m_reverse_symtab.insert(reverse_symtab_type::value_type(symbol, new_id));
	return new_id;
}

string
ConfLexer::getSymbol(int id)
{
	symtab_type::const_iterator fit = m_symtab.find(id);
	if (fit == m_symtab.end()) {
		// not found
		return "";
	}
	return fit->second;
}

void
ConfLexer::reset() {
	m_symtab.clear();
	m_reverse_symtab.clear();
	m_lineno = 1;
}

