%{
#include <string>
#include <iostream>
#include <sstream>
#include <algorithm>
#include "confparser.h"
#include "conflexer.h"

using namespace std;

static int section_num = -1;
static string section_str;
%}

%token TOK_SECTION TOK_EQUAL TOK_NEWLINE TOK_KEY TOK_VALUE TOK_OTHER

%%

config:
	lines
	;

lines: /* empty */
	| lines line
	;

line:
	section TOK_NEWLINE
	| pair TOK_NEWLINE
	| TOK_NEWLINE
	;
	

section:
	TOK_SECTION {
		section_num = $1;
		section_str = ConfLexer::getSymbol(section_num);
		ConfParser::processSection(section_str);
	}
	;
	
pair:
	TOK_KEY TOK_EQUAL TOK_VALUE {
		ConfParser::processParameter($1, $3);
	}
	;

%%

ConfParser::listeners_type ConfParser::m_listeners;
string ConfParser::m_fname;

void yyerror(const char *err)
{
	ConfParser::processError(err);
}

void
ConfParser::attachListener(ConfListener *listener)
{
	if (find(m_listeners.begin(), m_listeners.end(), listener) != m_listeners.end()) {
		// already attached
		return;
	}
	m_listeners.push_back(listener);
}

void
ConfParser::detachListener(ConfListener *listener)
{
	listeners_type::iterator fit;
	fit = find(m_listeners.begin(), m_listeners.end(), listener);
	if (fit == m_listeners.end()) {
		// not attached
		return;
	}
	m_listeners.erase(fit);
}

void
ConfParser::reset()
{
	section_num = -1;
	section_str = "";
}

bool
ConfParser::parse(FILE *input)
{
	ConfLexer::setInput(input);
	return (yyparse()==0);
}

void
ConfParser::processSection(const string& section)
{
	int line = ConfLexer::getCurLine();
	listeners_type::iterator iter = m_listeners.begin();
	for (; iter != m_listeners.end(); ++iter) {
		(*iter)->processSection(section, line);
	}
}

void
ConfParser::processParameter(int key, int value)
{
	string key_str = ConfLexer::getSymbol(key);
	string val_str = ConfLexer::getSymbol(value);
	int line = ConfLexer::getCurLine();
	listeners_type::iterator iter = m_listeners.begin();
	for (; iter != m_listeners.end(); ++iter) {
		(*iter)->processParameter(section_str, key_str, val_str, line);
	}
}

void
ConfParser::processError(const string& errmsg)
{
	int line = ConfLexer::getCurLine();
	listeners_type::iterator iter = m_listeners.begin();
	for (; iter != m_listeners.end(); ++iter) {
		(*iter)->processError(errmsg, line);
	}
}
