/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "decompressorresponsefilter.h"
#include "deflatedecompressor.h"
#include "gzipdecompressor.h"
#include "debug.h"

DecompressorResponseFilter::DecompressorResponseFilter(compr_type type)
:	m_type(type), m_metadata(0), m_eof(false)
{
	if (type == C_DEFLATE) {
		m_delegate = new DeflateDecompressor();
	} else {
		m_delegate = new GzipDecompressor();
	}
}

DecompressorResponseFilter::~DecompressorResponseFilter()
{
	delete m_delegate;
}

void
DecompressorResponseFilter::consumeMetadata(HttpResponseMetadata& metadata)
{
	m_metadata = &metadata;
	string enctype = (m_type == C_DEFLATE ? "deflate" : "gzip");
	HttpHeader *teh = m_metadata->headers().getHeaderPtr("Transfer-Encoding");
	HttpHeader *ceh = m_metadata->headers().getHeaderPtr("Content-Encoding");
	if (teh && !teh->elements().empty()) {
		if (teh->elements().getElements().back().getLowercaseName() == enctype) {
			teh->elements().removeLastElement();
			if (teh->elements().empty()) {
				m_metadata->headers().removeHeader("Transfer-Encoding");
			}
		} else {
			DEBUGLOG2("BUG");
		}
	} else if (ceh && !ceh->elements().empty()) {
		if (ceh->elements().getElements().back().getLowercaseName() == enctype) {
			ceh->elements().removeLastElement();
			if (ceh->elements().empty()) {
				m_metadata->headers().removeHeader("Content-Encoding");
			}
		} else {
			DEBUGLOG2("BUG");
		}
	} else {
		DEBUGLOG2("BUG");
	}
	m_metadata->setBodyStatus(HttpMessageMetadata::BODY_SIZE_UNKNOWN);
}

void
DecompressorResponseFilter::consumeDataChunk(const string& data)
{
	m_delegate->consume(data);
	string out;
	m_delegate->readOutput(out);
	m_processed_data += out;
}

void
DecompressorResponseFilter::consumeEOF()
{
	m_delegate->consumeEOF();
	string out;
	m_delegate->readOutput(out);
	m_processed_data += out;
	m_eof = true;
}

bool
DecompressorResponseFilter::hasDataAvailable() const
{
	return !m_processed_data.empty();
}

void
DecompressorResponseFilter::readDataChunk(string& collector)
{
	collector.assign(m_processed_data);
	m_processed_data.erase();
}

