/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <cstring>
#include "deflatedecompressor.h"

DeflateDecompressor::DeflateDecompressor() : m_error(false), m_eof(false), m_stream_finished(false)
{
	m_strm.zalloc = Z_NULL;
	m_strm.zfree = Z_NULL;
	m_strm.opaque = Z_NULL;
	m_strm.next_in = Z_NULL;
	inflateInit(&m_strm);
}

DeflateDecompressor::~DeflateDecompressor()
{
	inflateEnd(&m_strm);
}

void
DeflateDecompressor::reset()
{
	inflateEnd(&m_strm);
	m_strm.zalloc = Z_NULL;
	m_strm.zfree = Z_NULL;
	m_strm.opaque = Z_NULL;
	m_strm.next_in = Z_NULL;
	inflateInit(&m_strm);
	m_data.erase();
	m_error = false;
	m_eof = false;
	m_stream_finished = false;
}

bool
DeflateDecompressor::readOutput(string& output)
{
	output.erase();
	if (m_error)
		return false;
	if (m_data.empty())
		return true;
	bool res = doReadOutput(output);
	m_error = !res || (!m_data.empty() && (m_eof || m_stream_finished));
	return res;
}

bool
DeflateDecompressor::doReadOutput(string& output)
{
	int res;
	do {
		m_strm.next_in = m_inbuf;
		string::size_type indata_size = sizeof(m_inbuf)<m_data.length() ? sizeof(m_inbuf) : m_data.length();
		m_strm.avail_in = indata_size;
		memcpy(m_inbuf, m_data.c_str(), indata_size);
		do {
			m_strm.next_out = m_outbuf;
			m_strm.avail_out = sizeof(m_outbuf);
			res = inflate(&m_strm, Z_SYNC_FLUSH);
			output += string(m_outbuf, m_outbuf+sizeof(m_outbuf)-m_strm.avail_out);
		} while (res == Z_OK && m_strm.avail_out == 0);
		m_data.erase(0, indata_size-m_strm.avail_in);
	} while (res == Z_OK && !m_data.empty());
	if (res == Z_STREAM_END) {
		m_stream_finished = true;
		return true;
	} else if (m_eof) {
		return false;
	}
	return (res == Z_OK);
}

