/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef GZIPDECOMPRESSOR_H_
#define GZIPDECOMPRESSOR_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <zlib.h>
#include "abstractdecompressor.h"

using namespace std;

class GzipDecompressor : public AbstractDecompressor
{
public:
	GzipDecompressor();
	virtual ~GzipDecompressor();
	virtual void reset();
	virtual void consume(const string& data) { if (!m_error) m_data += data; }
	virtual void consumeEOF() { m_eof = true; }
	virtual bool readOutput(string& output);
private:
	enum state_enum {
		STATE_HEAD,
		STATE_FEXTRA,
		STATE_FNAME,
		STATE_FCOMMENT,
		STATE_FHCRC,
		STATE_DATA,
		STATE_CRC32_ISIZE
	};
	bool doReadOutput(string& output);
	bool m_error;
	bool m_eof;
	bool m_stream_ready;
	string m_data;
	state_enum m_state;
	unsigned char m_flg;
	unsigned long m_head_crc32; // these long's could be 32 or 64 bits, but it'll work either way
	unsigned long m_data_crc32;
	unsigned long m_data_length;
	z_stream m_strm;
	Bytef m_inbuf[4096];
	Bytef m_outbuf[8192];
};

#endif

