/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "utils.h"
#include "htmltaginfo.h"
#include "htmlelementnode.h"
#include "utils.h"

string
HTMLElementNode::toString(bool xhtml, const set<const HTMLNode *> *noclose) const
{
	string contents = contentsToString(xhtml, noclose);
	if (noclose && noclose->find(this) != noclose->end()) {
		return openingToString(xhtml, false)+contents;
	}
	
	if (contents.empty()) {
		if (xhtml) {
			return openingToString(xhtml, true)+contents;
		} else {
			string res = openingToString(xhtml, false)+contents;
			if (HTMLTagInfo::getInfoOn(m_name).m_etag_status != HTMLTagInfo::ETAG_FORBIDDEN) {
				res += string("</")+m_name+'>';
			}
			return res;
		}
	} else {
		return openingToString(xhtml, false)+contents+"</"+m_name+">";
	}
}

string
HTMLElementNode::contentsToString(bool xhtml, const set<const HTMLNode *> *noclose) const
{
	string res;
	list<HTMLNode *>::const_iterator iter = children().begin();
	for (; iter != children().end(); ++iter) {
		res += (*iter)->toString(xhtml, noclose);
	}
	return res;
}

string
HTMLElementNode::openingToString(bool xhtml, bool explicit_empty) const
{
	string res = "<";
	res += m_name;
	set<string>::const_iterator iter = m_attributes.begin();
	for (; iter != m_attributes.end(); iter++) {
		res += " ";
		res += *iter;
		if (isBooleanAttribute(*iter)) {
			if (xhtml) {
				res += "=\"yes\"";
			}
		} else {
			res += string("=\"")+str_replace("\"", "&quot;", getRawAttribute(*iter))+"\"";
		}
	}
	if (explicit_empty) {
		res += " />";
	} else {
		res += ">";
	}
	return res;
}

void
HTMLElementNode::setAttribute(const string& attr, const string& val)
{
	m_attributes.insert(attr);
	m_attr_values[attr] = htmlencode(val);
}

void
HTMLElementNode::setRawAttribute(const string& attr, const string& val)
{
	m_attributes.insert(attr);
	m_attr_values[attr] = val;
}

string
HTMLElementNode::getAttribute(const string& attr) const
{
	return htmldecode(getRawAttribute(attr));
}

string
HTMLElementNode::getRawAttribute(const string& attr) const
{
	set<string>::const_iterator fit = m_attributes.find(attr);
	if (fit == m_attributes.end())
		return "";
	map<string, string>::const_iterator fit2 = m_attr_values.find(attr);
	if (fit2 == m_attr_values.end())
		return "";
	return fit2->second;
}

bool
HTMLElementNode::isBooleanAttribute(const string& attr) const
{
	return (m_attributes.find(attr) != m_attributes.end()
		&& m_attr_values.find(attr) == m_attr_values.end());
}

HTMLElementNode*
HTMLElementNode::findAncestor(const string& name, bool include_self)
{
	if (include_self && m_name == name) {
		return this;
	}
	if (getParent()) {
		return ((HTMLElementNode *)getParent())->findAncestor(name, true);
	}
	return 0;
}

const HTMLElementNode*
HTMLElementNode::findAncestor(const string& name, bool include_self) const
{
	if (include_self && m_name == name) {
		return this;
	}
	if (getParent()) {
		return ((HTMLElementNode *)getParent())->findAncestor(name, true);
	}
	return 0;
}

