/*
    BFilter - a web proxy which removes banner ads
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include <sstream>
#include "lexgen.h"
#include "nfachar.h"
#include "nfacharclass.h"
#include "nfanegatedcharclass.h"
#include "nfaconcatenation.h"
#include "nfaclosure.h"
#include "nfastring.h"
#include "nfaistring.h"
#include "nfaunion.h"

#include "htmllexer.h"

int main()
{
	LexGen lexgen;
	lexgen.useSubclass("HTMLLexer");
	lexgen.includeCode("#include <htmllexer.h>\n");
	
	NFACharClass space(" \t\r\n");
	NFACharClass alpha('A', 'Z');
	alpha.addChars('a', 'z');
	NFACharClass ident = alpha;
	ident.addChars('0', '9');
	ident.addChars("-:");
	NFANegatedCharClass not_ident('0', '9');
	not_ident.addChars('a', 'z');
	not_ident.addChars('A', 'Z');
	not_ident.addChars("-:");
	NFANegatedCharClass any;
	NFAChar dquote('"');
	NFAChar squote('\'');
	NFAChar lbracket('<');
	NFAChar rbracket('>');
	
	NFAString comment_begin("<!--");
	NFAString comment_end("-->");
	
	NFAConcatenation cdata_end(NFAString("</"), alpha);
	cdata_end.addComponent(NFAClosure(ident));
	cdata_end.addComponent(rbracket);
	
	NFAIString script_end("</script>");
	NFAIString style_end("</style>");
	
	lexgen.addRule(HTMLLexer::INITIAL, NFAString("<!DOCTYPE"), "begin(INSIDE_DOCTYPE); more();", &space);
	lexgen.addRule(HTMLLexer::INITIAL, lbracket, "begin(TAG_NAME); setCurTag(tokText()); return TOK_TAG_BEGIN;", &alpha);
	lexgen.addRule(HTMLLexer::INITIAL, NFAString("</"), "begin(CLOSING_TAG_NAME); return TOK_CLOSING_TAG_BEGIN;", &alpha);
	lexgen.addRule(HTMLLexer::INITIAL, comment_begin, "begin(INSIDE_COMMENT); return TOK_COMMENT;");
	lexgen.addRule(HTMLLexer::INITIAL, any, "return TOK_TEXT;");
	lexgen.addRule(HTMLLexer::INSIDE_COMMENT, comment_end, "begin(INITIAL); return TOK_COMMENT;");
	lexgen.addRule(HTMLLexer::INSIDE_COMMENT, any, "return TOK_COMMENT;");
	lexgen.addRule(HTMLLexer::CDATA, cdata_end, "if (isCDATAEnding(tokText().substr(2, tokText().length()-3))) { begin(INITIAL); less(tokText().length()); } else return TOK_CDATA;");
	lexgen.addRule(HTMLLexer::CDATA, comment_begin, "begin(CDATA_COMM); return TOK_CDATA;");
	lexgen.addRule(HTMLLexer::CDATA, any, "return TOK_CDATA;");
	lexgen.addRule(HTMLLexer::CDATA_COMM, NFAConcatenation(NFAClosure(any), comment_end), "begin(CDATA); return TOK_CDATA;", 0, LexGen::OPT_LAZY);
	lexgen.addRule(HTMLLexer::CDATA_COMM, any, "begin(getCurTagName() == \"script\" ? SCRIPT_COMM_BROKEN : STYLE_COMM_BROKEN); less(1);");
	ostringstream strm1;
	strm1 << "begin(INITIAL); less(" << script_end.length() << "); return TOK_CDATA;";
	lexgen.addRule(HTMLLexer::SCRIPT_COMM_BROKEN, NFAConcatenation(NFAClosure(any), script_end), strm1.str());
	lexgen.addRule(HTMLLexer::SCRIPT_COMM_BROKEN, any, "return TOK_CDATA;");
	ostringstream strm2;
	strm2 << "begin(INITIAL); less(" << style_end.length() << "); return TOK_CDATA;";
	lexgen.addRule(HTMLLexer::STYLE_COMM_BROKEN, NFAConcatenation(NFAClosure(any), style_end), strm2.str());
	lexgen.addRule(HTMLLexer::STYLE_COMM_BROKEN, any, "return TOK_CDATA;");
	lexgen.addRule(HTMLLexer::INSIDE_DOCTYPE, NFAConcatenation(NFAClosure(any), rbracket), "begin(INITIAL); return TOK_DOCTYPE;", 0, LexGen::OPT_LAZY);
	lexgen.addRule(HTMLLexer::INSIDE_DOCTYPE, NFAConcatenation(NFAClosure(any), NFAChar('[')), "begin(INSIDE_DOCTYPE_INTERNAL_SUBSET); more();", 0, LexGen::OPT_LAZY);
	lexgen.addRule(HTMLLexer::INSIDE_DOCTYPE_INTERNAL_SUBSET, NFAConcatenation(NFAClosure(any), NFAChar(']')), "begin(INSIDE_DOCTYPE); more();", 0, LexGen::OPT_LAZY);
	lexgen.addRule(HTMLLexer::TAG_NAME, not_ident, "begin(INSIDE_TAG); less(1); curTagAppend(tokText()); setCurTagName(tokText()); return TOK_TAG_NAME;");
	lexgen.addRule(HTMLLexer::TAG_NAME, any, "more();");
	lexgen.addRule(HTMLLexer::CLOSING_TAG_NAME, not_ident, "begin(INSIDE_CLOSING_TAG); less(1); return TOK_CLOSING_TAG_NAME;");
	lexgen.addRule(HTMLLexer::CLOSING_TAG_NAME, any, "more();");
	lexgen.addRule(HTMLLexer::INSIDE_TAG, NFAConcatenation(alpha, NFAClosure(ident)), "begin(ATTR_EQUAL); curTagAppend(tokText()); return TOK_ATTR_NAME;");
	lexgen.addRule(HTMLLexer::ATTR_EQUAL, NFAConcatenation(NFAClosure(space), NFAChar('=')), "begin(ATTR_VALUE); curTagAppend(tokText());");
	lexgen.addRule(HTMLLexer::ATTR_EQUAL, any, "begin(INSIDE_TAG); less(1);");
	lexgen.addRule(HTMLLexer::ATTR_VALUE, dquote, "begin(ATTR_VALUE_DQ); curTagAppend(tokText());");
	lexgen.addRule(HTMLLexer::ATTR_VALUE, squote, "begin(ATTR_VALUE_SQ); curTagAppend(tokText());");
	lexgen.addRule(HTMLLexer::ATTR_VALUE, NFAClosure(space), "curTagAppend(tokText());");
	lexgen.addRule(HTMLLexer::ATTR_VALUE, any, "more(); begin(ATTR_VALUE_NQ);");
	lexgen.addRule(HTMLLexer::ATTR_VALUE_NQ, NFACharClass(" \t\n\r>"), "begin(INSIDE_TAG); less(1); curTagAppend(tokText()); return TOK_ATTR_VALUE;");
	lexgen.addRule(HTMLLexer::ATTR_VALUE_NQ, any, "more();");
	lexgen.addRule(HTMLLexer::ATTR_VALUE_DQ, NFAClosure(NFANegatedCharClass("\"")), "begin(INSIDE_TAG); curTagAppend(tokText()); return TOK_ATTR_VALUE;", &dquote);
	lexgen.addRule(HTMLLexer::ATTR_VALUE_DQ, any, "more(); begin(ATTR_VALUE_NQ);");
	lexgen.addRule(HTMLLexer::ATTR_VALUE_SQ, NFAClosure(NFANegatedCharClass("'")), "begin(INSIDE_TAG); curTagAppend(tokText()); return TOK_ATTR_VALUE;", &squote);
	lexgen.addRule(HTMLLexer::ATTR_VALUE_SQ, any, "more(); begin(ATTR_VALUE_NQ);");
	lexgen.addRule(HTMLLexer::INSIDE_TAG, NFAChar('/'), "curTagAppend(tokText()); return TOK_TAG_EMPTY;", &rbracket);
	lexgen.addRule(HTMLLexer::INSIDE_TAG, rbracket, "curTagAppend(tokText()); begin(isCDATAStarting()?CDATA:INITIAL); return TOK_TAG_END;");
	lexgen.addRule(HTMLLexer::ATTR_EQUAL, rbracket, "begin(INITIAL); curTagAppend(tokText()); return TOK_TAG_END;");
	lexgen.addRule(HTMLLexer::ATTR_VALUE, rbracket, "begin(INITIAL); curTagAppend(tokText()); return TOK_TAG_END;");
	lexgen.addRule(HTMLLexer::ATTR_VALUE_DQ, rbracket, "begin(INITIAL); curTagAppend(tokText()); return TOK_TAG_END;");
	lexgen.addRule(HTMLLexer::ATTR_VALUE_SQ, rbracket, "begin(INITIAL); curTagAppend(tokText()); return TOK_TAG_END;");
	lexgen.addRule(HTMLLexer::INSIDE_CLOSING_TAG, rbracket, "begin(INITIAL); return TOK_CLOSING_TAG_END;");
	lexgen.addRule(HTMLLexer::INSIDE_TAG, any, "curTagAppend(tokText());");
	lexgen.addRule(HTMLLexer::ATTR_VALUE, any, "curTagAppend(tokText());");
	lexgen.addRule(HTMLLexer::ATTR_VALUE_DQ, any, "curTagAppend(tokText());");
	lexgen.addRule(HTMLLexer::ATTR_VALUE_SQ, any, "curTagAppend(tokText());");
	lexgen.addRule(HTMLLexer::INSIDE_CLOSING_TAG, any, "");
	
	lexgen.generateLexer(cout);
}

