/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef HTMLNODE_H_
#define HTMLNODE_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <list>
#include <algorithm>

using namespace std;

class HTMLNode
{
	friend class HTMLStack;
public:
	enum node_type_enum { NODE_TEXT, NODE_ELEMENT };
	HTMLNode() : m_parent(0) {}
	virtual ~HTMLNode() {}
	virtual node_type_enum getType() const = 0;
	virtual string toString(bool xhtml, const set<const HTMLNode *> *noclose=0) const = 0;
	virtual string contentsToString(bool xhtml, const set<const HTMLNode *> *noclose=0) const = 0;
	virtual HTMLNode *getParent() { return m_parent; }
	virtual const HTMLNode *getParent() const { return m_parent; }
	virtual list<HTMLNode *> getChildren() const { return m_children; }
protected:
	virtual HTMLNode* clone() const = 0;
	virtual const list<HTMLNode *>& children() const { return m_children; }
	virtual void appendChild(HTMLNode *node)
	{
		removeChild(node);
		m_children.push_back(node);
		node->m_parent = this;
	}
	virtual void removeChild(HTMLNode *node)
	{
		list<HTMLNode *>::iterator fit = find(m_children.begin(), m_children.end(), node);
		if (fit != m_children.end()) {
			(*fit)->m_parent = 0;
			m_children.erase(fit);
		}
	}
	virtual void removeAllChildren()
	{
		list<HTMLNode *>::iterator iter = m_children.begin();
		for (; iter != m_children.end(); ++iter) {
			(*iter)->m_parent = 0;
		}
		m_children.clear();
	}
private:
	HTMLNode *m_parent;
	list<HTMLNode *> m_children;
};

#endif

