/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "htmlresponsefilter.h"

HTMLResponseFilter::HTMLResponseFilter(HTMLProcessor& processor)
:	m_processor(processor), m_metadata(0), m_eof(false)
{}

HTMLResponseFilter::~HTMLResponseFilter()
{
	m_processor.reset();
}

void
HTMLResponseFilter::consumeMetadata(HttpResponseMetadata& metadata)
{
	m_metadata = &metadata;
	m_metadata->setBodyStatus(HttpMessageMetadata::BODY_SIZE_UNKNOWN);
}

void
HTMLResponseFilter::consumeDataChunk(const string& data)
{
	m_processor.consumeData(data);
}

void
HTMLResponseFilter::consumeEOF()
{
	m_processor.consumeEOF();
	m_eof = true;
}

bool
HTMLResponseFilter::hasDataAvailable() const
{
	return !m_processor.peekProcessedData().empty();
}

void
HTMLResponseFilter::readDataChunk(string& collector)
{
	collector.assign(m_processor.readProcessedData());
}

