/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef HTMLSTACK_H_
#define HTMLSTACK_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <set>
#include "htmlnode.h"
#include "htmltextnode.h"
#include "htmlelementnode.h"

using namespace std;

class HTMLStack
{
public:
	HTMLStack();
	HTMLStack(const HTMLStack& stack);
	~HTMLStack();
	HTMLStack& operator=(const HTMLStack& stack);
	HTMLElementNode *top() { return m_top; }
	HTMLElementNode *bottom() { return m_bottom; }
	void appendElement(const HTMLElementNode& el);
	void appendText(const string& text);
	void clearTop(); // deletetes all children from the top element
	void pushElement(const HTMLElementNode& el); // the new element becomes the new top
	void popElement(); // removes an element from the top
	void walkOn(); // swithes m_top to the point to its parent
	void trimTo(const HTMLElementNode* el);
	void clear();
	string toString(bool xhtml) const;
private:
	void copyInClearEnvironment(const HTMLStack& stack);
	HTMLElementNode *m_top; // the innermost element
	HTMLElementNode *m_bottom;
	set<HTMLNode *> m_nodes;
};

#endif

