/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef HTTPCLIENTREQUEST_H_
#define HTTPCLIENTREQUEST_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <list>
#include "uri.h"
#include "httprequestmetadata.h"
#include "namevaluepair.h"

using namespace std;

class HttpClientRequest
{
public:
	enum InterceptHandlerType { IH_NONE, IH_IMAGE_SUBSTITUTE, IH_IMAGE_ANALYZE, IH_ERROR_SUBSTITUTE };
	class Listener
	{
		friend class HttpClientRequest;
	public:
		virtual ~Listener() {}
	protected:
		virtual void moreBodyData(const string& data) = 0;
		virtual void complete() = 0;
	};
	HttpClientRequest(const HttpRequestMetadata& metadata);
	void setListener(Listener& listener) { m_listener = &listener; }
	void removeListener() { m_listener = 0; }
	HttpRequestMetadata& metadata() { return m_metadata; }
	const HttpRequestMetadata& metadata() const { return m_metadata; }
	const HttpRequestLine& getOrigRequestLine() const { return m_orig_request_line; }
	const URI& getAbsoluteURI() const { return m_absolute_uri; }
	const string& getBodyData() const { return m_body_data; }
	void appendBodyData(const string& data);
	bool isComplete() const { return m_is_complete; }
	void setComplete();
	bool isAlreadyServed() const { return m_is_already_served; }
	void setAlreadyServed() { m_is_already_served = true; }
	bool isClosingRequest() const { return m_is_closing; }
	bool isToBeSent() const;
	bool isIdempotent() const;
	InterceptHandlerType getInterceptHandler() const { return m_intercept_handler; }
	void setInterceptHandler(InterceptHandlerType ih) { m_intercept_handler = ih; }
	const list<NameValuePair>& getInterceptParams() const { return m_intercept_params; }
	string getInterceptParam(const string& name) const;
	const string& getErrorResponse() const { return m_error_response; }
	void setErrorResponse(const string& error_response) { m_error_response = error_response; }
private:
	Listener *m_listener;
	HttpRequestMetadata m_metadata;
	HttpRequestLine m_orig_request_line;
	URI m_absolute_uri;
	string m_body_data; // possibly incomplete
	bool m_is_closing;
	bool m_is_complete;
	bool m_is_already_served;
	InterceptHandlerType m_intercept_handler;
	list<NameValuePair> m_intercept_params;
	string m_error_response;
};

#endif

