/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "httpheaderscollection.h"

#include <string>
#include <map>
#include "utils.h"
#include "httpheader.h"


void
HttpHeadersCollection::setHeader(const HttpHeader& header)
{
	m_headers[header.getLowercaseName()] = header;
}

HttpHeader
HttpHeadersCollection::getHeader(const string& name) const
{
	headers_type::const_iterator fiter = m_headers.find(strtolower(name));
	if (fiter == m_headers.end()) {
		return HttpHeader();
	}
	return fiter->second;
}

HttpHeader *
HttpHeadersCollection::getHeaderPtr(const string& name)
{
	headers_type::iterator fiter = m_headers.find(strtolower(name));
	if (fiter == m_headers.end()) {
		return 0;
	}
	return &fiter->second;
}

void
HttpHeadersCollection::removeHeader(const string& name)
{
	m_headers.erase(strtolower(name));
}

bool
HttpHeadersCollection::hasHeader(const string& name) const
{
	return (m_headers.find(strtolower(name)) != m_headers.end());
}

string
HttpHeadersCollection::toString() const
{
	string res;
	headers_type::const_iterator iter = m_headers.begin();
	for (; iter != m_headers.end(); iter++) {
		res += iter->second.toString();
	}
	return res;
}

