/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef HTTPREQUESTLINE_H_
#define HTTPREQUESTLINE_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include "httpstartline.h"
#include "httpversion.h"
#include "uri.h"

using namespace std;

class HttpRequestLine : public HttpStartLine
{
public:
	HttpRequestLine(const string& method, const URI& uri, const HttpVersion& http_version)
	: m_method(method), m_uri(uri), m_http_version(http_version) {}
	const string& getMethod() const { return m_method; }
	void setMethod(const string& method) { m_method = method; }
	const URI& getURI() const { return m_uri; }
	void setURI(const URI& uri) { m_uri = uri; }
	virtual const HttpVersion& getHttpVersion() const { return m_http_version; }
	virtual void setHttpVersion(const HttpVersion& version) { m_http_version = version; }
	virtual string toString() const;
private:
	string m_method;
	URI m_uri;
	HttpVersion m_http_version;
};

inline string
HttpRequestLine::toString() const
{
	return m_method+" "+m_uri.toString()+" "+m_http_version.toString();
}

#endif

