/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "httpstateflatbody.h"

#include <string>
#include "httpstream.h"

using namespace std;

HttpStateFlatBody::HttpStateFlatBody(HttpStream& owner)
:	m_owner(owner), m_body_size_defined(false), m_body_size(0), m_read(0)
{}

void
HttpStateFlatBody::consumeDataChunk(string& pool)
{
	string::size_type would_be_read = m_read+pool.length();
	if (m_body_size_defined && would_be_read >= m_body_size) {
		if (would_be_read == m_body_size) {
			m_owner.eventGotContentPart(pool);
			pool.assign("");
		} else {
			m_owner.eventGotContentPart(string(pool, 0, m_body_size-m_read));
			pool.erase(0, m_body_size-m_read);
		}
		m_owner.switchState(HttpStream::STATE_INITIAL);
		m_owner.eventMessageEnds(m_owner.m_connection_needs_closing);
		return;
	} else {
		m_owner.eventGotContentPart(pool);
		m_read += pool.length();
		pool.assign("");
	}
}

void
HttpStateFlatBody::consumeEOF()
{
	if (m_body_size_defined) {
		m_owner.eventError(HttpStream::ERR_UNEXPECTED_EOF);
		m_owner.jumpToState(HttpStream::STATE_ERROR);
	} else {
		m_owner.switchState(HttpStream::STATE_INITIAL);
		m_owner.eventMessageEnds(true);
	}
}

void
HttpStateFlatBody::cleanup()
{
	m_body_size_defined = false;
	m_body_size = 0;
	m_read = 0;
}

void
HttpStateFlatBody::setBodySize(string::size_type size)
{
	m_body_size_defined = true;
	m_body_size = size;
}

