/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef HTTPSTREAM_H_
#define HTTPSTREAM_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include "httpstate.h"
#include "httpstatestartline.h"
#include "httpstateheaders.h"
#include "httpstateinitial.h"
#include "httpstateflatbody.h"
#include "httpstatechunkheader.h"
#include "httpstatechunkbody.h"
#include "httpstatefooters.h"
#include "httpstatesimpleresponse.h"
#include "httpstateerror.h"
#include "httpheaderscollection.h"
#include "httprequestmetadata.h"
#include "httpresponsemetadata.h"
#include "httpversion.h"

using namespace std;

class HttpStream
{
	friend class HttpStateInitial;
	friend class HttpStateStartLine;
	friend class HttpStateHeaders;
	friend class HttpStateFlatBody;
	friend class HttpStateChunkHeader;
	friend class HttpStateChunkBody;
	friend class HttpStateFooters;
	friend class HttpStateSimpleResponse;
	friend class HttpStateError;
public:
	HttpStream();
	virtual ~HttpStream() {}
	void consumeDataChunk(const string& data);
	void consumeEOF();
	void resetState();
	bool isInitialState() const;
	bool isErrorState() const;
protected:
	enum ErrorCode {
		ERR_PARSING_REQUEST_LINE = 0,
		ERR_PARSING_HEADER,
		ERR_PARSING_CHUNK_HEADER,
		ERR_LENGTH_REQUIRED,
		ERR_UNEXPECTED_EOF,
		NUMBER_OF_ERRORS
	};
	virtual bool isRequestStream() const = 0;
	virtual bool isResponseOnHeadRequest() const = 0;
	virtual void eventError(int ecode, const char *info=0) = 0;
	virtual void eventGotMetadata(const HttpRequestMetadata& metadata) = 0;
	virtual void eventGotMetadata(const HttpResponseMetadata& metadata) = 0;
	virtual void eventGotContentPart(const string& data) = 0;
	virtual void eventGotFooters(const HttpHeadersCollection& footers) = 0;
	virtual void eventMessageEnds(bool connection_needs_closing) = 0;
private:
	enum State {
		STATE_INITIAL = 0,
		STATE_START_LINE,
		STATE_HEADERS,
		STATE_FLAT_BODY,
		STATE_CHUNK_HEADER,
		STATE_CHUNK_BODY,
		STATE_FOOTERS,
		STATE_SIMPLE_RESPONSE,
		STATE_ERROR,
		NUMBER_OF_STATES
	};
	typedef HttpState * HttpStatePtr;
	class ChangeStateException
	{
	public:
		ChangeStateException(State state) : m_state(state) {}
		State getState() const { return m_state; }
	private:
		State m_state;
	};
	HttpStream(const HttpStream& other);             //
	HttpStream& operator=(const HttpStream& other);  // not implemented
	void switchState(State state);
	void jumpToState(State state);
	HttpStateInitial m_state_initial;
	HttpStateStartLine m_state_start_line;
	HttpStateHeaders m_state_headers;
	HttpStateFlatBody m_state_flat_body;
	HttpStateChunkHeader m_state_chunk_header;
	HttpStateChunkBody m_state_chunk_body;
	HttpStateFooters m_state_footers;
	HttpStateSimpleResponse m_state_simple_response;
	HttpStateError m_state_error;
	HttpState *m_state;
	HttpStatePtr m_state_table[NUMBER_OF_STATES];
	bool m_inside_consume_fn;
	bool m_connection_needs_closing;
};

#endif

