/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "imageresponsefilter.h"
#include "httpheader.h"
#include "htmlprocessor.h"
#include "replacementimage.h"
#include "conf.h"
#include "utils.h"

ImageResponseFilter::ImageResponseFilter(int ad_rule, int width, int height, const string& target,
	bool external_link, bool external_img, bool suspicious_link, bool suspicious_img, bool redir, bool cookies)
:	m_state(ST_INSPECTING), m_eof(false), m_metadata(0),
	m_ad_rule(ad_rule), m_width(width), m_height(height), m_target(target),
	m_external_link(external_link), m_external_img(external_img),
	m_suspicious_link(suspicious_link), m_suspicious_img(suspicious_img), m_redir(redir), m_cookies(cookies)
{}

void
ImageResponseFilter::consumeDataChunk(const string& data)
{
	if (m_eof) {
		return;
	}
	if (m_state == ST_INSPECTING) {
		m_inspector.consumeData(data);
		if (!m_inspector.isInspectionInProgress()) {
			if (isAd()) {
				m_state = ST_SUBSTITUTING;
				int width = m_width;
				int height = m_height;
				if (width == 0)
					width = m_inspector.getWidth();
				if (height == 0)
					height = m_inspector.getHeight();
				ReplacementImage image(width, height, Conf::getInstance().drawBorders(), Conf::getInstance().getBorderColor());
				m_metadata->headers().setHeader(HttpHeader("Content-Type", image.getContentType()));
				m_metadata->setBodyStatus(HttpMessageMetadata::BODY_SIZE_KNOWN);
				m_metadata->setBodySize(image.getBinaryData().length());
				m_processed_data.assign(image.getBinaryData());
				m_eof = true;
			} else {
				m_state = ST_FORWARDING;
				m_processed_data = m_inspector.getAccumulatedData();
			}
		}
	} else if (m_state == ST_FORWARDING) {
		m_processed_data += data;
	}
	// do nothing if m_state == ST_SUBSTITUTING
}

void
ImageResponseFilter::consumeEOF()
{
	if (m_state == ST_INSPECTING) {
		// broken or unsupported image
		m_state = ST_FORWARDING;
		m_processed_data = m_inspector.getAccumulatedData();
	}
	m_eof = true;
}

bool
ImageResponseFilter::isMetadataReady() const
{
	return (m_state != ST_INSPECTING);
}

bool
ImageResponseFilter::hasDataAvailable() const
{
	return !m_processed_data.empty();
}

void
ImageResponseFilter::readDataChunk(string& collector)
{
	collector.assign(m_processed_data);
	m_processed_data.assign("");
}

bool
ImageResponseFilter::isAd() const
{
	int res = HTMLProcessor::imageIsAd(m_ad_rule, m_inspector.getWidth(), m_inspector.getHeight(),
		m_target, m_external_link, m_external_img, m_suspicious_link, m_suspicious_img,
		(m_inspector.isAnimated() ? 1 : -1), (m_redir ? 1 : -1), (m_cookies ? 1 : -1));
	if (res == 1) {
		return true;
	}
	return false;
}
