/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef IPCMESSAGE_H_
#define IPCMESSAGE_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <iostream>
#include <sstream>
#include <vector>

using namespace std;

class IPCMessage
{
public:
	IPCMessage() {}
	IPCMessage(const string& method) : m_method(method) {}
	const string& getMethod() const { return m_method; }
	void setMethod(const string& method) { m_method = method; }
	void addParameter(const string& param) { m_params.push_back(param); }
	void addParameter(int param)
	{ ostringstream strm; strm << param; addParameter(strm.str()); }
	void clearParams() { m_params.clear(); }
	int nParams() const { return m_params.size(); }
	string getParameter(int pos) const { return (pos < nParams() ? m_params[pos] : ""); }
	int getParameterAsInt(int pos) const
	{ int res = 0; istringstream strm(getParameter(pos)); strm >> res; return res; }
	string serialize() const;
	void unserialize(const string& str);
private:
	string readBytes(istream& strm, string::size_type bytes);
	string m_method;
	vector<string> m_params;
};

#endif
