/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef IPCPORTAL_H_
#define IPCPORTAL_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <queue>
#include <unistd.h>
#include "ipcmessage.h"

using namespace std;

class IPCPortal
{
public:
	IPCPortal() : m_rfd(-1), m_wfd(-1), m_receiving_header(true) {}
	IPCPortal(int rfd, int wfd) : m_rfd(rfd), m_wfd(wfd), m_receiving_header(true) {}
	int nReceived() const { return m_receive_queue.size(); }
	const IPCMessage *firstReceived() const;
	bool hasDataToSend() const { return !m_to_send.empty(); }
	IPCMessage readMessage();
	void sendMessage(const IPCMessage& msg);
	void readChunk(const string& data);
	bool readChunk();
	bool writeChunk();
	int getReadFD() const { return m_rfd; }
	int getWriteFD() const { return m_wfd; }
private:
	int m_rfd, m_wfd;
	queue<IPCMessage> m_receive_queue;
	queue<IPCMessage> m_send_queue;
	string m_to_send;
	bool m_receiving_header;
	string m_rec_header;
	string::size_type m_rec_data_size;
	string m_rec_data;
};

#endif

