/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <windows.h>
#include <shellapi.h>
#include <winsock2.h>
#include <string>
#include <fstream>
#include "state.h"
#include "conf.h"
#include "rules.h"
#include "network.h"
#include "utils.h"
#include "errorreporter.h"
#include "childfinishexception.h"
#include "bfilter.h"
#include "resources/resources.h"


using namespace std;

#define UWM_SYSTRAY (WM_USER + 1)
#define UWM_RESTART (WM_USER + 2)
#define UWM_EXIT    (WM_USER + 3)

namespace Globals {
	string basePath;
	HANDLE hMainMutex = NULL;
	HINSTANCE hInstance = NULL;
	HWND hMainWnd = NULL;
	HWND hConfWnd = NULL;
	Network::Socket serverSocket = Network::INVALID_SOCK;
}

struct Config
{
	string listen_addr;
	int listen_port;
	bool client_compression;
	bool draw_borders;
	unsigned char border_color[3];
	bool use_aux_proxy;
	string proxy_host;
	int proxy_port;
};

LRESULT CALLBACK WindowProcedure(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK ConfDialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
DWORD WINAPI MainNetworkThread(LPVOID lParam);
DWORD WINAPI ChildNetworkThread(LPVOID lParam);
void InstallTrayIcon();
void RemoveTrayIcon();
bool WriteConfig(const Config& conf);


int WINAPI
WinMain(HINSTANCE hThisInstance, HINSTANCE hPrevInstance,
        LPSTR lpCmdLine, int nCmdShow)

{
	HWND hwnd;
	MSG msg;
	WNDCLASSEX wincl;
	
	char module_fname[512];
	GetModuleFileName(NULL, module_fname, sizeof(module_fname));
	string module_path(module_fname);
	string::size_type pos = module_path.rfind('\\');
	if (pos != string::npos) {
		module_path.erase(pos+1);
	} else {
		return 0;
	}
	Globals::basePath = module_path;
	Globals::hInstance = hThisInstance;
	Conf::setConfDir(Globals::basePath+"conf");
	Rules::setConfDir(Globals::basePath+"conf");
	
	if (string(lpCmdLine).find("--fix-config") != string::npos) {
		Conf::getInstance().readConfig();
		Config conf;
		conf.listen_addr = Conf::getInstance().getListenAddress();
		conf.listen_port = Conf::getInstance().getListenPort();
		conf.client_compression = Conf::getInstance().clientCompression();
		conf.draw_borders = Conf::getInstance().drawBorders();
		memcpy(conf.border_color, Conf::getInstance().getBorderColor(), 3);
		conf.use_aux_proxy = Conf::getInstance().usingProxy();
		conf.proxy_host = Conf::getInstance().proxyHost();
		conf.proxy_port = Conf::getInstance().proxyPort();
		WriteConfig(conf);
		return 0;
	}
	
	Globals::hMainMutex = CreateMutex(NULL, FALSE, "Local\\InstanceMutex");
	if (!Globals::hMainMutex) {
		ErrorReporter::fatalError("Mutex creation failed");
	}
	if (GetLastError() == ERROR_ALREADY_EXISTS) {
		return 0;
	}
	
	WORD wVersionRequested = MAKEWORD(2, 0);
	WSADATA wsaData;
	if (WSAStartup(wVersionRequested, &wsaData) != 0) {
		ErrorReporter::fatalError("WSAStartup() failed");
	}
	
	Conf::getInstance().readConfig();
	Rules::getInstance().readRules();
	
	NetworkError err;
	Globals::serverSocket = Network::tcpServer(Conf::getInstance().getListenAddress().c_str(),
	                                           Conf::getInstance().getListenPort(), &err);
	if (!err.isOK()) {
		ErrorReporter::fatalError(err.getErrorMessage());
	}
	
	// Main window - never shown
	wincl.hInstance = Globals::hInstance;
	wincl.lpszClassName = "BFilter";
	wincl.lpfnWndProc = WindowProcedure;
	wincl.style = CS_DBLCLKS;
	wincl.cbSize = sizeof(WNDCLASSEX);
	wincl.hIcon = LoadIcon(NULL, MAKEINTRESOURCE(IDI_APP_ICON));
	wincl.hIconSm = LoadIcon(NULL, MAKEINTRESOURCE(IDI_APP_ICON));
	wincl.hCursor = LoadCursor(NULL, IDC_ARROW);
	wincl.lpszMenuName = NULL;
	wincl.cbClsExtra = 0;
	wincl.cbWndExtra = 0;
	wincl.hbrBackground = (HBRUSH)COLOR_BACKGROUND;

	if (!RegisterClassEx(&wincl))
		return 0;

	hwnd = CreateWindowEx(0,                   /* Extended possibilites for variation */
	                      "BFilter",           /* Classname */
	                      "BFilter",           /* Title Text */
	                      WS_OVERLAPPEDWINDOW, /* default window */
	                      CW_USEDEFAULT,       /* Windows decides the position */
	                      CW_USEDEFAULT,       /* where the window ends up on the screen */
	                      544, 375,            /* The programs width and height in pixels */
	                      HWND_DESKTOP,        /* The window is a child-window to desktop */
	                      NULL,                /* No menu */
	                      Globals::hInstance,  /* Program Instance handler */
	                      NULL                 /* No Window Creation data */
	);
	Globals::hMainWnd = hwnd;
	
	DWORD dwThreadId;
	HANDLE hThread = CreateThread(NULL, 0, MainNetworkThread, NULL, 0, &dwThreadId);
	if (hThread == NULL) {
		PostMessage(Globals::hMainWnd, UWM_EXIT, 0, 0);
	} else {
		CloseHandle(hThread); // we don't need it
	}
	
	InstallTrayIcon();
	
	while (GetMessage(&msg, NULL, 0, 0)) {
		if (!IsWindow(Globals::hConfWnd) || !IsDialogMessage(Globals::hConfWnd, &msg)) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
	
	Network::closeSocket(Globals::serverSocket);
	WSACleanup();
	if (Globals::hMainMutex)
		CloseHandle(Globals::hMainMutex);
	
	return msg.wParam;
}

LRESULT CALLBACK
WindowProcedure(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
	case WM_DESTROY:
	case UWM_EXIT:
		RemoveTrayIcon();
		PostQuitMessage(0);
		break;
	case UWM_SYSTRAY:
		switch (lParam) {
		case WM_RBUTTONUP:
		case WM_LBUTTONUP:
			POINT pt;
			GetCursorPos(&pt);
			HMENU hmenu = LoadMenu(Globals::hInstance, "TrayMenu");
			HMENU hpopup = GetSubMenu(hmenu, 0);
			CheckMenuItem(hpopup, IDM_FILTER,
			              State::getInstance().getFiltering() ? MF_CHECKED : MF_UNCHECKED);
			CheckMenuItem(hpopup, IDM_FILTER_SCRIPTS,
			              State::getInstance().getScriptFiltering() ? MF_CHECKED : MF_UNCHECKED);
			EnableMenuItem(hpopup, IDM_FILTER_SCRIPTS,
			               State::getInstance().getFiltering() ? MF_ENABLED : MF_GRAYED);
			SetForegroundWindow(hwnd);
			while (true) {
				BOOL tpm_res = TrackPopupMenu(hpopup, TPM_RETURNCMD|TPM_RIGHTBUTTON,
				                              pt.x, pt.y, 0, hwnd, NULL);
				switch (tpm_res) {
				case IDM_FILTER: {
					bool turn_on = (CheckMenuItem(hpopup, IDM_FILTER, MF_BYCOMMAND) == MF_UNCHECKED);
					State::getInstance().setFiltering(turn_on);
					CheckMenuItem(hpopup, IDM_FILTER, turn_on ? MF_CHECKED : MF_UNCHECKED);
					EnableMenuItem(hpopup, IDM_FILTER_SCRIPTS, turn_on ? MF_ENABLED : MF_GRAYED);
					CheckMenuItem(hpopup, IDM_FILTER_SCRIPTS,
					              (turn_on && State::getInstance().getScriptFiltering()) ?
					              MF_CHECKED : MF_UNCHECKED);
					continue;
				}
				case IDM_FILTER_SCRIPTS: {
					bool checked = (CheckMenuItem(hpopup, IDM_FILTER_SCRIPTS, MF_BYCOMMAND) == MF_CHECKED);
					State::getInstance().setScriptFiltering(!checked);
					CheckMenuItem(hpopup, IDM_FILTER_SCRIPTS, checked ? MF_UNCHECKED : MF_CHECKED);
					continue;
				}
				case IDM_CONFIGURE:
					if (IsWindow(Globals::hConfWnd))
						break;
					Globals::hConfWnd = CreateDialog(
						Globals::hInstance,
						MAKEINTRESOURCE(IDD_OPTIONS_DIALOG),
						Globals::hMainWnd, (DLGPROC)ConfDialogProc
					);
					break;
				case IDM_EXIT:
					DestroyWindow(hwnd);
					break;
				}
				break;
			}
			PostMessage(hwnd, WM_NULL, 0, 0);
			DestroyMenu(hmenu);
			break;
		}
		break;
	case UWM_RESTART: {
		CloseHandle(Globals::hMainMutex);
		Globals::hMainMutex = NULL;
		char module_fname[512];
		GetModuleFileName(NULL, module_fname, sizeof(module_fname));
		STARTUPINFO si;
		PROCESS_INFORMATION pi;
		ZeroMemory(&si, sizeof(si));
		si.cb = sizeof(si);
		ZeroMemory(&pi, sizeof(pi));
		CreateProcess(module_fname, module_fname, 0, 0, FALSE, 0, NULL, NULL, &si, &pi);
		CloseHandle(pi.hProcess);
		CloseHandle(pi.hThread);
		PostMessage(Globals::hMainWnd, UWM_EXIT, 0, 0);
		break;
	}
	default:
		return DefWindowProc(hwnd, message, wParam, lParam);
	}

	return 0;
}

LRESULT CALLBACK
ConfDialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
	case WM_INITDIALOG:
		SetDlgItemInt(hDlg, IDC_LISTEN_PORT, Conf::getInstance().getListenPort(), TRUE);
		CheckDlgButton(hDlg, IDC_USE_HTTP_PROXY, Conf::getInstance().usingProxy() ? BST_CHECKED : BST_UNCHECKED);
		SetDlgItemText(hDlg, IDC_HTTP_PROXY_HOST, Conf::getInstance().proxyHost().c_str());
		if (Conf::getInstance().proxyPort() < 0) {
			SetDlgItemText(hDlg, IDC_HTTP_PROXY_PORT, "");
		} else {
			SetDlgItemInt(hDlg, IDC_HTTP_PROXY_PORT, Conf::getInstance().proxyPort(), TRUE);
		}
		return TRUE;
	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK) {
			Config conf;
			conf.listen_addr = Conf::getInstance().getListenAddress();
			TCHAR buf[300];
			GetDlgItemText(hDlg, IDC_LISTEN_PORT, buf, sizeof(buf)/sizeof(buf[0]));
			conf.listen_port = str2num<int>(buf, -1);
			if (conf.listen_port < 1 || conf.listen_port > 65535) {
				MessageBox(hDlg, "Listen Port must be an integer in the range of [1, 65535]", "Error", MB_OK|MB_ICONERROR);
				return TRUE;
			}
			conf.client_compression = Conf::getInstance().clientCompression();
			conf.draw_borders = Conf::getInstance().drawBorders();
			memcpy(conf.border_color, Conf::getInstance().getBorderColor(), 3);
			conf.use_aux_proxy = (IsDlgButtonChecked(hDlg, IDC_USE_HTTP_PROXY)==BST_CHECKED);
			GetDlgItemText(hDlg, IDC_HTTP_PROXY_HOST, buf, sizeof(buf)/sizeof(buf[0]));
			conf.proxy_host = buf;
			GetDlgItemText(hDlg, IDC_HTTP_PROXY_PORT, buf, sizeof(buf)/sizeof(buf[0]));
			conf.proxy_port = str2num<int>(buf, -1);
			if (conf.use_aux_proxy && (conf.proxy_port < 1 || conf.proxy_port > 65535)) {
				MessageBox(hDlg, "Proxy Port must be an integer in the range of [1, 65535]", "Error", MB_OK|MB_ICONERROR);
				return TRUE;
			}
			if (conf.listen_port == Conf::getInstance().getListenPort() &&
			    conf.use_aux_proxy == Conf::getInstance().usingProxy() &&
			    conf.proxy_host == Conf::getInstance().proxyHost() &&
			    conf.proxy_port == Conf::getInstance().proxyPort()) {
				// no changes
				DestroyWindow(hDlg);
				Globals::hConfWnd = NULL;
				return TRUE;
			}
			WriteConfig(conf);
			DestroyWindow(hDlg);
			Globals::hConfWnd = NULL;
			if (MessageBox(Globals::hMainWnd, "Restart BFilter?", "Question", MB_YESNO|MB_ICONQUESTION)==IDYES) {
				PostMessage(Globals::hMainWnd, UWM_RESTART, 0, 0);
			}
			DestroyWindow(hDlg);
			Globals::hConfWnd = NULL;
			return TRUE;
		} else if (LOWORD(wParam) == IDCANCEL) {
			DestroyWindow(hDlg);
			Globals::hConfWnd = NULL;
			return TRUE;
		}
		break;
	}
	return FALSE;
}

DWORD WINAPI MainNetworkThread(LPVOID lParam)
{
	Network::Socket csock;
	struct sockaddr_in client;
	while (true) {
		csock = Network::tcpServerWaitConn(Globals::serverSocket, &client);
		if (!Network::isValidSocket(csock))
			continue;
		DWORD dwThreadId;
		HANDLE hThread = CreateThread(NULL, 0, ChildNetworkThread, (LPVOID)csock, 0, &dwThreadId);
		if (hThread) {
			CloseHandle(hThread); // we don't need it
		}
	}
	return 0;
}

DWORD WINAPI ChildNetworkThread(LPVOID lParam)
{
	Network::Socket csock = (Network::Socket)lParam;
	Network::sockSetNodelay(csock, true);
	try {
		BFilter bfilter(csock);
		bfilter.run();
	} catch (ChildFinishException& e) {}
	Network::disconnectAndCloseSocket(csock, 10);
	return 0;
}

void InstallTrayIcon()
{
	NOTIFYICONDATA nid;
	nid.cbSize = sizeof(NOTIFYICONDATA);
	nid.hWnd = Globals::hMainWnd;
	nid.uID = 1;
	nid.uFlags = NIF_MESSAGE | NIF_ICON;
	nid.uCallbackMessage = UWM_SYSTRAY;
	nid.hIcon = LoadIcon(Globals::hInstance, MAKEINTRESOURCE(IDI_TRAY_ICON));
	Shell_NotifyIcon(NIM_ADD, &nid);
}

void RemoveTrayIcon()
{
	NOTIFYICONDATA nid;
	nid.cbSize = sizeof(NOTIFYICONDATA);
	nid.hWnd = Globals::hMainWnd;
	nid.uID = 1;
	nid.uFlags = 0;
	Shell_NotifyIcon(NIM_DELETE, &nid);
}

bool WriteConfig(const Config& conf) {
	const string CONFIG_FILE = Globals::basePath+"conf\\config";
	ofstream strm(CONFIG_FILE.c_str());
	if (!strm.good()) {
		return false;
	}
	strm << "[global]" << endl;
	strm << "listen_address=" << conf.listen_addr << ":" << conf.listen_port << endl;
	strm << "client_compression=" << (conf.client_compression ? "yes" : "no") << endl;
	if (conf.draw_borders) {
		char buf[10];
		snprintf(buf, sizeof(buf), "%02x%02x%02x", conf.border_color[0],
		         conf.border_color[1], conf.border_color[2]);
		strm << "ad_border=" << buf << endl;
	} else {
		strm << "ad_border=none" << endl;
	}
	strm << endl;
	strm << "[forwarding]" << endl;
	strm << "use_proxy=" << (conf.use_aux_proxy ? "yes" : "no") << endl;
	strm << "proxy_host=" << conf.proxy_host << endl;
	strm << "proxy_port=" << (conf.proxy_port < 0 ? "" : num2str(conf.proxy_port).c_str()) << endl;
	return strm.good();
}

