/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef SERVERRESPONSESTREAM_H_
#define SERVERRESPONSESTREAM_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include "httpstream.h"
#include "httpstreamwriter.h"
#include "responsefilterchain.h"
#include "httpclientrequest.h"

using namespace std;

class BFilter;

class ServerResponseStream : public HttpStream, protected HttpStreamWriter::Receiver
{
public:
	ServerResponseStream(BFilter& owner)
	: m_owner(owner), m_filter_chain(0), m_stream_writer(0), m_forwarding_in_progress(false) {}
	virtual ~ServerResponseStream();
	bool isForwardingInProgress() { return m_forwarding_in_progress; }
	void processSubstitutionRequest(const HttpClientRequest *request); // to be called from ClientRequestStream and locally
protected:
	/* Inherited from HttpStream */
	virtual bool isRequestStream() const { return false; }
	virtual bool isResponseOnHeadRequest() const;
	virtual void eventError(int ecode, const char *info=0);
	virtual void eventGotMetadata(const HttpResponseMetadata& metadata);
	virtual void eventGotContentPart(const string& data);
	virtual void eventGotFooters(const HttpHeadersCollection& footers) {}
	virtual void eventMessageEnds(bool connection_needs_closing);
	
	/* Inherited from HttpStreamWriter::Receiver */
	virtual void moreData(const string& data);
	virtual void done(bool conn_close);
private:
	ServerResponseStream(const ServerResponseStream& stream);            //
	ServerResponseStream& operator=(const ServerResponseStream& stream); // not implemented
	virtual void eventGotMetadata(const HttpRequestMetadata& metadata) {}
	static void prepareResponseForForwarding(HttpResponseMetadata& metadata, const HttpClientRequest *active_request);
	void addFilterToChain(HttpResponseMetadata& metadata, AbstractResponseFilter *filter);
	void performImageSubstitution(const HttpClientRequest *request);
	void performErrorSubstitution(const HttpClientRequest *request);
	BFilter& m_owner;
	ResponseFilterChain *m_filter_chain;
	HttpStreamWriter *m_stream_writer;
	bool m_forwarding_in_progress;
	int m_response_status_code;
};

#endif

